@@ -1,25 +1,19 @@
 /* Common routines & arrays. */
 
 #include "mod3.h"
+
+#include "../deck.h"
 #include "../mod3.h"
 #include "../pile.h"
-#include "../deck.h"
 
-#include <cstdio>
-#include <cstdlib>
-#include <cstring>
-#include <cctype>
-#include <cmath>
-#include <sys/types.h>
-#include <cstdarg>
+#include <KDebug>
 
-#include <kdebug.h>
-
-/* These two routines make and unmake moves. */
 
 #define PRINT 0
 #define PRINT2 0
 
+/* These two routines make and unmake moves. */
+
 void Mod3Solver::make_move(MOVE *m)
 {
 #if PRINT
