@@ -11,22 +11,19 @@
  * event will the author be liable for any lost revenue or profits or
  * other special, indirect and consequential damages.
 */
-#include "pile.h"
-#include "dealer.h"
-#include "cardmaps.h"
-#include "speeds.h"
-#include "deck.h"
 
-#include <cassert>
+#include "pile.h"
+
+#include "cardmaps.h"
+#include "dealer.h"
+#include "deck.h"
+#include "render.h"
+#include "speeds.h"
+
+#include <KDebug>
+
 #include <cmath>
 
-#include <QSvgRenderer>
-#include <QPainter>
-
-#include <kstandarddirs.h>
-#include <kdebug.h>
-
-const int Pile::my_type       = DealerScene::PileTypeId;
 
 const int Pile::Default       = 0x0000;
 const int Pile::disallow      = 0x0001;
@@ -40,16 +37,14 @@ const int Pile::autoTurnTop   = 0x0200;
 const int Pile::wholeColumn   = 0x0400;
 const int Pile::demoOK        = 0x0800;
 
-QPixmap *Pile::cache = 0;
-QPixmap *Pile::cache_selected = 0;
-
 Pile::Pile( int _index, DealerScene* parent)
     : QGraphicsPixmapItem( ),
       _atype(Custom),
       _rtype(Custom),
       myIndex(_index),
       _target(false),
-      m_highlighted( false )
+      m_highlighted( false ),
+      m_graphicVisible( true )
 {
     setObjectName( "<unknown>" );
     parent->addItem( this );
@@ -66,24 +61,10 @@ Pile::Pile( int _index, DealerScene* parent)
     setSpread( 3.3 );
     setReservedSpace( QSizeF( 10, 10 ) );
     setShapeMode( QGraphicsPixmapItem::BoundingRectShape );
-    setMaximalSpace( QSizeF( 1, 1 ) ); // just to make it valid
+    setMaximumSpace( QSizeF( 1, 1 ) ); // just to make it valid
     m_relayoutTimer = new QTimer( this );
     m_relayoutTimer->setSingleShot( true );
-    connect( m_relayoutTimer, SIGNAL( timeout() ), SLOT( relayoutCards() ) );
-
-    if ( !cache )
-        cache = new QPixmap( KStandardDirs::locateLocal( "data", "kpat/pile.png" ) );
-    if ( !cache_selected )
-        cache_selected = new QPixmap( KStandardDirs::locateLocal( "data", "kpat/pile_selected.png" ) );
-}
-
-QSvgRenderer *Pile::_renderer = 0;
-
-QSvgRenderer *Pile::pileRenderer()
-{
-    if ( !_renderer )
-        _renderer = new QSvgRenderer( KStandardDirs::locate( "data", "kpat/pile.svg" ) );
-    return _renderer;
+    connect( m_relayoutTimer, SIGNAL(timeout()), SLOT(relayoutCards()) );
 }
 
 DealerScene *Pile::dscene() const
@@ -167,20 +148,15 @@ void Pile::rescale()
     if (!scene())
         return;
 
-    int wantedheight = int( cardMap::self()->wantedCardHeight() + 1 );
-    QPixmap *mycache = cache;
-    if ( isHighlighted() )
-        mycache = cache_selected;
-
-    QPointF new_pos = QPointF( _pilePos.x() * cardMap::self()->wantedCardWidth() / 10.,
-                               _pilePos.y() * cardMap::self()->wantedCardHeight() / 10. );
-    //kDebug() << scene()->sceneRect() << pos() << new_pos << dscene()->offsetX();
+    QPointF new_pos = QPointF( _pilePos.x() * cardMap::self()->cardWidth() / 10.,
+                               _pilePos.y() * cardMap::self()->cardHeight() / 10. );
+    //kDebug(11111) << scene()->sceneRect() << pos() << new_pos << dscene()->offsetX();
     if ( new_pos.x() < 0 )
-        new_pos.setX( scene()->width() - cardMap::self()->wantedCardWidth() + new_pos.x() );
+        new_pos.setX( scene()->width() - cardMap::self()->cardWidth() + new_pos.x() );
     else
         new_pos.rx() += dscene()->offsetX();
     if ( new_pos.y() < 0 )
-        new_pos.setY( scene()->height() - cardMap::self()->wantedCardHeight() + new_pos.y() );
+        new_pos.setY( scene()->height() - cardMap::self()->cardHeight() + new_pos.y() );
 
     if ( new_pos != pos() )
     {
@@ -188,37 +164,17 @@ void Pile::rescale()
         tryRelayoutCards();
     }
 
-    if ( mycache->height() == wantedheight )
+    QSize size = cardMap::self()->cardSize();
+    if ( m_graphicVisible )
     {
-        setPixmap( *mycache );
-        return;
+        setPixmap( Render::renderElement( isHighlighted() ? "pile_selected" : "pile", size ) );
     }
-    //kDebug(11111) << gettime() << "rescale start\n";
-
-#if 0
-    QImage pix( ( int )maximalSpace().width(),
-                ( int )maximalSpace().height(),
-                QImage::Format_ARGB32 );
-    pix.fill( qRgba( 0, 0, 255, 255 ) );
-#else
-    QImage pix( static_cast<int>( cardMap::self()->wantedCardWidth() + 1 ),
-                static_cast<int>( cardMap::self()->wantedCardHeight() + 1),
-                QImage::Format_ARGB32 );
-    pix.fill( qRgba( 0, 0, 255, 0 ) );
-#endif
-    QPainter p( &pix );
-
-    pileRenderer()->render( &p, isHighlighted() ? "pile_selected" : "pile",
-                            QRectF( 0, 0, cardMap::self()->wantedCardWidth(),
-                                    cardMap::self()->wantedCardHeight() ) );
-    p.end();
-    *mycache = QPixmap::fromImage( pix );
-    if ( isHighlighted() )
-        pix.save( KStandardDirs::locateLocal( "data", "kpat/pile_selected.png" ), "PNG" );
     else
-        pix.save( KStandardDirs::locateLocal( "data", "kpat/pile.png" ), "PNG" );
-    setPixmap( *mycache );
-    //kDebug(11111) << gettime() << "rescale end\n";
+    {
+        QPixmap blank( size );
+        blank.fill( Qt::transparent );
+        setPixmap( blank );
+    }
 }
 
 bool Pile::legalAdd( const CardList& _cards, bool ) const
@@ -301,7 +257,7 @@ void Pile::setVisible(bool vis)
 
 int Pile::indexOf(const Card *c) const
 {
-    assert(c->source() == this);
+    Q_ASSERT(c->source() == this);
     return m_cards.indexOf(const_cast<Card*>(c)); // the list is of non-const cards
 }
 
@@ -336,45 +292,17 @@ void Pile::relayoutCards()
 {
     m_relayoutTimer->stop();
 
-    QPointF mypos = pos();
-    qreal z = zValue() + 1;
-    QSizeF preferredSize = QSizeF( 0, 0 );
-
-    for (CardList::Iterator it = m_cards.begin(); it != m_cards.end(); ++it)
+    foreach ( Card * card, m_cards )
     {
-        //kDebug() << ( *it )->name() << " " << ( *it )->spread();
-        // the spreads should hopefully have all one sign, or we get in trouble
-        preferredSize.rwidth() += fabs( ( *it )->spread().width() );
-        preferredSize.rheight() += fabs( ( *it )->spread().height() );
-        if ( ( *it )->animated() || dscene()->isMoving( *it ) ) {
-            // kDebug() << ( *it )->name() << "is animated - restarting timer\n";
+        if ( card->animated() || dscene()->isMoving( card ) ) {
             tryRelayoutCards();
             return;
         }
     }
-    qreal divy = 1;
-    qreal divx = 1;
-    if ( preferredSize.height() )
-        divy = qMin( ( maximalSpace().height() - cardMap::self()->wantedCardHeight() ) / preferredSize.height() * 10 / cardMap::self()->wantedCardHeight(), 1. );
-    if ( preferredSize.width() )
-        divx = qMin( ( maximalSpace().width() - cardMap::self()->wantedCardWidth() ) / preferredSize.width() * 10 / cardMap::self()->wantedCardWidth(), 1. );
 
-    //kDebug() << "relayoutCards" << objectName() << " " <<  divx << " " << divy << " " << maximalSpace() << " " << preferredSize << " " << reservedSpace();
-
-    for (CardList::Iterator it = m_cards.begin(); it != m_cards.end(); ++it)
-    {
-        // ( *it )->stopAnimation();
-        ( *it )->moveTo( mypos.x(), mypos.y(), z, dscene()->speedUpTime(  120 ) );
-        ( *it )->setZValue( z );
-        mypos.rx() += ( *it )->spread().width() * divx / 10 * cardMap::self()->wantedCardWidth();
-        mypos.ry() += ( *it )->spread().height() * divy / 10 * cardMap::self()->wantedCardHeight();
-        z += 1;
-    }
+    layoutCards( dscene()->speedUpTime( DURATION_RELAYOUT ) );
 }
 
-#include <sys/time.h>
-#include <stdio.h>
-
 void Pile::add( Card *_card, int index)
 {
     if (_card->source() == this)
@@ -398,7 +326,7 @@ void Pile::add( Card *_card, int index)
     else {
         while (m_cards.count() <= index)
             m_cards.append(0);
-        assert(m_cards[index] == 0);
+        Q_ASSERT(m_cards[index] == 0);
         m_cards[index] = _card;
     }
 
@@ -444,9 +372,9 @@ void Pile::add( Card* _card, bool _facedown )
     qreal x2, y2, z2;
 
     if (t) {
-        // kDebug() << "::add" << t->pos() << " " << t->spread() << " " << _card->name() << " " << t->name() << " " << _card->spread();
-        x2 = t->realX() + t->spread().width()  / 10 * cardMap::self()->wantedCardWidth();
-        y2 = t->realY() + t->spread().height() / 10 * cardMap::self()->wantedCardHeight();
+        // kDebug(11111) << "::add" << t->pos() << " " << t->spread() << " " << _card->name() << " " << t->name() << " " << _card->spread();
+        x2 = t->realX() + t->spread().width()  / 10 * cardMap::self()->cardWidth();
+        y2 = t->realY() + t->spread().height() / 10 * cardMap::self()->cardHeight();
         z2 = t->realZ() + 1;
     } else {
         x2 = x();
@@ -470,7 +398,7 @@ void Pile::add( Card* _card, bool _facedown )
     } else {
         if ( source == Deck::deck() && dscene()->isInitialDeal() )
         {
-            _card->setPos(QPointF( x2, 0 - 3 * cardMap::self()->wantedCardHeight() ) );
+            _card->setPos(QPointF( x2, 0 - 3 * cardMap::self()->cardHeight() ) );
         }
         _card->setZValue( z2 );
         qreal distx = x2 - _card->x();
@@ -482,7 +410,7 @@ void Pile::add( Card* _card, bool _facedown )
         if ( _card->animated() )
         {
             dscene()->setWaiting( true );
-            connect(_card, SIGNAL(stoped(Card*)), SLOT(waitForMoving(Card*)));
+            connect(_card, SIGNAL(stopped(Card*)), SLOT(waitForMoving(Card*)));
         }
     }
 }
@@ -495,7 +423,7 @@ void Pile::waitForMoving( Card*c )
 
 void Pile::remove(Card *c)
 {
-    assert(m_cards.contains(c));
+    Q_ASSERT(m_cards.contains(c));
     m_cards.removeAll(c);
 }
 
@@ -516,11 +444,17 @@ void Pile::setHighlighted( bool flag ) {
     rescale();
 }
 
+void Pile::setGraphicVisible( bool flag ) {
+    m_graphicVisible = flag;
+    rescale();
+}
+
+
 CardList Pile::cardPressed(Card *c)
 {
     CardList result;
 
-    //kDebug() << gettime() << "cardPressed" << c->name() << " " << c->isFaceUp();
+    //kDebug(11111) << gettime() << "cardPressed" << c->name() << " " << c->isFaceUp();
     emit pressed( c );
 
     if ( !legalRemove(c) )
@@ -564,53 +498,15 @@ void Pile::moveCards(CardList &cl, Pile *to)
     to->moveCardsBack(cl);
 }
 
-void Pile::moveCardsBack(CardList &cl, int steps )
+void Pile::moveCardsBack(CardList &cl, int duration)
 {
     if (!cl.count())
         return;
 
-    Card *c = cl.first();
+    if ( duration == -1 )
+        duration = DURATION_MOVEBACK;
 
-    Card *before = 0;
-    QSizeF off;
-
-    if ( steps == -1 )
-        steps = DURATION_MOVEBACK;
-
-    for (CardList::Iterator iti = m_cards.begin(); iti != m_cards.end(); ++iti)
-    {
-        if (c == *iti) {
-
-            if (before) {
-                // kDebug() << "moveCardsBack" << ( *iti )->name() << " " << before->name() << " " << before->spread();
-                off = before->spread();
-                c->moveTo( before->realX() + off.width() / 10 * cardMap::self()->wantedCardWidth(),
-			   before->realY() + off.height()  / 10 * cardMap::self()->wantedCardHeight(),
-			   before->realZ() + 1, steps);
-            }
-            else {
-                // kDebug() << "moveCardsBack" << ( *iti )->name() << "no before";
-                c->moveTo( x(), y(), zValue() + 1, steps);
-            }
-            break;
-        } else
-            before = *iti;
-    }
-
-    before = c;
-    CardList::Iterator it = cl.begin(); // == c
-    ++it;
-
-    for (; it != cl.end(); ++it)
-    {
-        off = before->spread();
-        (*it)->moveTo( before->realX() + off.width() / 10 * cardMap::self()->wantedCardWidth(),
-                       before->realY() + off.height()  / 10 * cardMap::self()->wantedCardHeight(),
-                       before->realZ() + 1, steps);
-        before = *it;
-    }
-
-    relayoutCards();
+    layoutCards( duration );
 }
 
 bool Pile::cardClicked(Card *c)
@@ -634,4 +530,32 @@ void Pile::tryRelayoutCards()
         m_relayoutTimer->start( 40 );
 }
 
+void Pile::layoutCards(int duration)
+{
+    QSizeF preferredSize( 0, 0 );
+    foreach ( const Card * card, m_cards )
+    {
+        // The spreads should hopefully have all one sign, or we get in trouble
+        preferredSize.rwidth() += fabs( card->spread().width() );
+        preferredSize.rheight() += fabs( card->spread().height() );
+    }
+
+    qreal divy = 1;
+    if ( preferredSize.height() )
+        divy = qMin<qreal>( ( maximumSpace().height() - cardMap::self()->cardHeight() ) / preferredSize.height() * 10 / cardMap::self()->cardHeight(), 1. );
+    qreal divx = 1;
+    if ( preferredSize.width() )
+        divx = qMin<qreal>( ( maximumSpace().width() - cardMap::self()->cardWidth() ) / preferredSize.width() * 10 / cardMap::self()->cardWidth(), 1. );
+
+    QPointF cardPos = pos();
+    qreal z = zValue() + 1;
+    foreach ( Card * card, m_cards )
+    {
+        card->moveTo( cardPos.x(), cardPos.y(), z, dscene()->speedUpTime( duration ) );
+        cardPos.rx() += divx * card->spread().width() / 10 * cardMap::self()->cardWidth();
+        cardPos.ry() += divy * card->spread().height() / 10 * cardMap::self()->cardHeight();
+        z += 1;
+    }
+}
+
 #include "pile.moc"
