@@ -35,13 +35,16 @@
 #include <kdebug.h>
 #include <klocale.h>
 #include <kmessagebox.h>
+#include <QUuid>
+#include <QMutexLocker>
+#include <QMap>
 
 
 /* Includes for the STL */
-#include <utility>
-#include <map>
+// #include <utility>
+// #include <map>
 
-#include <stdlib.h>
+// #include <stdlib.h>
 
 namespace Ksirk
 {
@@ -73,6 +76,12 @@ AIPlayer :: AIPlayer(
 //   kDebug() << "AIPlayer constructor" << endl;
 }
 
+AIPlayer::~AIPlayer()
+{
+  kDebug() << name();
+  m_thread.terminate();
+  m_thread.wait();
+}
 
 /**
   * This function is called whenever the player should choose an action (
@@ -81,7 +90,7 @@ AIPlayer :: AIPlayer(
   */
 void AIPlayer::actionChoice(GameLogic::GameAutomaton::GameState state)
 {
-  kDebug() << name() << ": (state is " << m_game-> stateName() << ")" << endl;
+  kDebug() << name() << ": (state is " << m_game-> stateName() << ", current player is "<<m_game-> currentPlayer()->name()<<")" << endl;
   if (m_game->m_aicannotrunhack)
   {
     kDebug() << "HACK HACK AIPlayer " << name()   
@@ -92,9 +101,9 @@ void AIPlayer::actionChoice(GameLogic::GameAutomaton::GameState state)
   {
     return;
   }
-  if (m_waitedAck != 0)
+  if (!m_waitedAck.isEmpty())
   {
-    kDebug() << Player::name() << " waiting to receive ack " << m_waitedAck << endl;
+    kDebug() << Player::name() << " waiting to receive ack " << m_waitedAck;
     return;
   }
   QPointF point;
@@ -102,10 +111,11 @@ void AIPlayer::actionChoice(GameLogic::GameAutomaton::GameState state)
   QDataStream stream(&buffer, QIODevice::WriteOnly);
   QByteArray buffer2;
   QDataStream stream2(&buffer2, QIODevice::WriteOnly);
-  if ( (m_game-> currentPlayer() == this) || (state == GameLogic::GameAutomaton::WAITDEFENSE && (m_game-> currentPlayer() != this))
-       ||  state == GameLogic::GameAutomaton::WAIT_RECYCLING)
+  if ( (m_game-> currentPlayer() == this)
+    || (state == GameLogic::GameAutomaton::WAITDEFENSE && (m_game-> currentPlayer() != this))
+    ||  (state == GameLogic::GameAutomaton::WAIT_RECYCLING && m_game-> currentPlayer()->isVirtual() ) )
   {
-    kDebug() << name()  << " : choosing my action" << endl;
+    kDebug() << name()  << " : choosing my action";
     switch (state)
     {
       case GameLogic::GameAutomaton::WAITDEFENSE :
@@ -186,6 +196,7 @@ void AIPlayer::actionChoice(GameLogic::GameAutomaton::GameState state)
       default :;
 //         kDebug() << "OTHER STATE:" << state << " "  << name() << endl;
     }
+    requestAck();
     if (m_game-> currentPlayer() != this)
       m_actionWaitingStart = false;
   }
@@ -235,14 +246,14 @@ void AIPlayer::chooseAttackMoveArmiesOrNextPlayer()
   */
 std::pair<const Country*, const Country*> AIPlayer::chooseBelligerant()
 {
-  std::multimap<const Country*, const Country*> candidates;
+  QMultiMap<const Country*, const Country*> candidates;
 
 //     kDebug() << name() << " : AIPlayer::chooseBelligerant()" << endl;
     // Builds the list of countries of the player that have enough armies and a good neighbour
-  std::vector< Country* > list = countries();
+  QList<Country*> list = countries();
   if ( ! list.empty() )
   {
-    std::vector< Country* >::iterator outer = list.begin();
+    QList<Country*>::iterator outer = list.begin();
     const Country* candidateSource;
 //        kDebug() << name() << "  choosing belligerants, candidate sources " << endl;
     while ( ( outer != list.end()) && ( (candidateSource = *outer) != 0 ) )
@@ -253,7 +264,7 @@ std::pair<const Country*, const Country*> AIPlayer::chooseBelligerant()
       {
 //                kDebug() << name() << "  choosing belligerants, candidate source has enough armies." << endl;
 //                kDebug() << name() << "  choosing belligerants, candidate targets " << endl;
-        for ( unsigned int inner = 0; inner < m_world-> getCountries().size(); inner++)
+        for ( int inner = 0; inner < m_world-> getCountries().size(); inner++)
         {
           const Country* candidateTarget = m_world-> getCountries().at(inner);
 //                    kDebug() << name() << "  choosing belligerants, looking at candidate target : " << candidateTarget-> name() << endl;
@@ -265,7 +276,7 @@ std::pair<const Country*, const Country*> AIPlayer::chooseBelligerant()
           {
 //                        kDebug() << name() << "  choosing belligerants, adding target / source : "
 //                                << candidateSource-> name() << " / " << candidateTarget-> name() << endl;
-              candidates.insert(std::make_pair(candidateSource, candidateTarget));
+              candidates.insert(candidateSource, candidateTarget);
           }
         }
       }
@@ -278,14 +289,14 @@ std::pair<const Country*, const Country*> AIPlayer::chooseBelligerant()
     }
     uint which = Dice::roll(candidates.size()) - 1;
 //        kDebug() << "Which = " << which <<endl;
-    std::multimap<const Country*, const Country*>::const_iterator it;
+    QMultiMap<const Country*, const Country*>::const_iterator it;
     unsigned int i = 0;
-    for ( it = candidates.begin(); it != candidates.end() ; it++, i++ )
+    for ( it = candidates.constBegin(); it != candidates.constEnd() ; it++, i++ )
     {
       if (which == i )
       {
 //                kDebug() << "OUT AIPlayer::chooseBelligerant() : " << endl ;
-        return (*it);
+         return std::make_pair(it.key(), it.value());
       }
     }
   }
@@ -301,15 +312,15 @@ std::pair<const Country*, const Country*> AIPlayer::chooseBelligerant()
 Country* AIPlayer::chooseReceivingCountry()
 {
 //     kDebug() << "AIPlayer::chooseReceivingCountry()" << endl;
-  std::vector<Country*> myCountries = countries();
+  QList<Country*> myCountries = countries();
   if (myCountries.size() == 0)
   {
     kError() << "AIPlayer::chooseReceivingCountry() EMPTY LIST" << endl;
     return 0;
   }
-  std::vector<Country*> withNeighbours;
+  QList<Country*> withNeighbours;
   
-  for (unsigned int i = 0 ; i < myCountries.size(); i++)
+  for (int i = 0 ; i < myCountries.size(); i++)
   {
     if ( ( m_world-> neighboursNotBelongingTo(*(myCountries.at(i)), static_cast< const Player * >(this) ) ).size() )
         withNeighbours.push_back(myCountries.at(i));
@@ -336,12 +347,12 @@ bool AIPlayer::isAI() const
 
 void AIPlayer::MyThread::run()
 {
-  kDebug();
+  kDebug() << me.name();
   stopMe = false;
   while ( ! stopMe )
   {
     me.actionChoice(me.m_game->state());
-    mssleep( 500 );
+    msleep( 500 );
   }
   kDebug() << "OUT";
 }
@@ -415,12 +426,11 @@ bool AIPlayer::attackAction()
       m_thread.exit();
   }
   aiPlayerIO()->sendInput(stream3,true);
-  /*QByteArray buffer2;
-  QDataStream stream2(&buffer2, QIODevice::WriteOnly);
-  stream2 << QString("actionLButtonDown") << m_src->centralPoint();
-  aiPlayerIO()->sendInput(stream2,true);*/
+//   requestAck();
+
   kDebug() << "AIPlayer " << Player::name()  << " : attackAction : "  << m_src-> name() << " " << m_dest-> name()
     << " " << nbAttack << endl;
+  stop();
   return true;
 }
 
@@ -430,7 +440,7 @@ bool AIPlayer::attackAction()
 bool AIPlayer::moveArmiesAction()
 {
   kDebug() << "AIPlayer::moveArmiesAction" << endl;
-  std::vector<Country*> srcList = countries() ;
+  QList<Country*> srcList = countries() ;
   if (srcList.size() == 0) 
   {
     nextPlayerAction(); 
@@ -445,7 +455,7 @@ bool AIPlayer::moveArmiesAction()
     return false;
   }
 //    kDebug() << "AIPlayer::moveArmiesAction() MOVEARMIES 2" << endl;
-  std::vector<Country*> destList( m_world-> neighboursBelongingTo(*osrc, this) );
+  QList<Country*> destList( m_world-> neighboursBelongingTo(*osrc, this) );
 //    kDebug() << "AIPlayer::moveArmiesAction() MOVEARMIES 3" << endl;
   if (destList.size() == 0) 
   {
@@ -467,7 +477,8 @@ bool AIPlayer::moveArmiesAction()
   QDataStream stream2(&buffer2, QIODevice::WriteOnly);
   stream2 << QString("actionLButtonUp") << odest->centralPoint();
   aiPlayerIO()->sendInput(stream2,true);
-
+//   requestAck();
+  
   kDebug() << "AIPlayer ****************" << Player::name()  << " : moveAction : "  << osrc-> name() << " " << odest-> name() << endl;
 
   return true;
@@ -485,9 +496,11 @@ void AIPlayer::placeArmiesAction()
     const Country* receiver = chooseReceivingCountry();
     if (receiver == 0)
     {
-      QString msg = i18n("Error - No receiving country selected while computer player %1 had still %2 armies to place. This is bug probably #2232 at www.gna.org.", Player::name(), getNbAvailArmies());
+      QString msg = i18np("Error - No receiving country selected while computer player %2 had still 1 army to place. This is bug probably #2232 at www.gna.org.", "Error - No receiving country selected while computer player %2 had still %1 armies to place. This is bug probably #2232 at www.gna.org.", getNbAvailArmies(), Player::name());
+      kError() << msg;
       KMessageBox::error(0, msg, i18n("Fatal Error"));
       m_thread.exit();
+      m_thread.wait();
     }
     kDebug() << "Placing an army in " << receiver->name() 
         << " ; point=" << receiver->centralPoint() << endl;
@@ -496,13 +509,7 @@ void AIPlayer::placeArmiesAction()
     stream << QString("actionLButtonDown") << receiver->centralPoint();
     aiPlayerIO()->sendInput(stream,true);
 
-    m_waitedAck = ++m_uid;
-    QByteArray buffer2;
-    QDataStream stream2(&buffer2, QIODevice::WriteOnly);
-    QPointF p(m_waitedAck, m_waitedAck);
-    kDebug() << name() << " sending a request for ack " << m_waitedAck << endl;
-    stream2 << QString("requestForAck") << p;
-    aiPlayerIO()->sendInput(stream2,true);
+//     requestAck();
   }
   else if (m_game->state() != GameAutomaton::INTERLUDE)
   {
@@ -511,17 +518,9 @@ void AIPlayer::placeArmiesAction()
     QPointF point;
     m_game->gameEvent("actionNextPlayer", point);
   }
-  else if (m_game->allLocalPlayersComputer())
+  else
   {
-    if (!m_hasVoted)
-    {
-      QByteArray buffer;
-      QDataStream stream(&buffer, QIODevice::WriteOnly);
-      QPointF point;
-      stream << QString("slotRecyclingFinished") << point;
-      aiPlayerIO()->sendInput(stream,true);
-      m_hasVoted = true;
-    }
+    chooseWetherToRecycle();
   }
   
 }
@@ -531,17 +530,14 @@ void AIPlayer::placeArmiesAction()
  */
 void AIPlayer::chooseWetherToRecycle() 
 {
-  kDebug() << "AIPlayer::chooseWetherToRecycle" << Player::name() << endl;
+  kDebug() << Player::name();
   if (m_game->allLocalPlayersComputer())
   {
     if (!m_hasVoted)
     {
       kDebug() << "Voting for end of recycling" << endl;
-      QByteArray buffer;
-      QDataStream stream(&buffer, QIODevice::WriteOnly);
-      QPointF point;
-      stream << QString("slotRecyclingFinished") << point;
-      aiPlayerIO()->sendInput(stream,true);
+      QPointF p;
+      m_game->gameEvent( "actionRecyclingFinished", p );
       m_hasVoted = true;
     }
     else
@@ -587,6 +583,7 @@ void AIPlayer::chooseInvasionAction()
   stream << QString("actionInvasionFinished") << point;
   stop();
   aiPlayerIO()->sendInput(stream,true);
+//   requestAck();
 }
 
 /**
@@ -625,6 +622,7 @@ void AIPlayer::chooseDefenseAction()
     }
     stop();
     aiPlayerIO()->sendInput(stream,true);
+//     requestAck();
   }
 }
 
@@ -639,6 +637,7 @@ void AIPlayer::nextPlayerAction()
   QDataStream stream(&buffer, QIODevice::WriteOnly);
   stream << QString("actionNextPlayer") << point;
   aiPlayerIO()->sendInput(stream,true);
+//   requestAck();
   stop();
 }
 
@@ -703,6 +702,18 @@ AIPlayerIO* AIPlayer::aiPlayerIO()
   return 0;
 }
 
+void AIPlayer::requestAck()
+{
+  QMutexLocker locker(&m_waitedAckMutex);
+  m_waitedAck = QUuid::createUuid().toString();
+  QByteArray buffer;
+  QDataStream stream(&buffer, QIODevice::WriteOnly);
+  QPointF p;
+  kDebug() << name() << " sending a request for ack " << m_waitedAck << endl;
+  stream << QString("requestForAck") << p << m_waitedAck;
+  aiPlayerIO()->sendInput(stream,true);
+}
+
 } // closing namespace GameLogic
 } // closing namespace Ksirk
 
